@@ -20,6 +20,7 @@
 #include "screenshooter-dialogs.h"
 #include "screenshooter-actions.h"
 #include "screenshooter-custom-actions.h"
+#include "screenshooter-format.h"
 
 #include <exo/exo.h>
 
@@ -578,29 +579,38 @@ static gchar
   GError *error = NULL;
   gchar *save_path = g_file_get_path (save_file);
   const char *type = "png";
+  char** option_keys = NULL;
+  char** option_values = NULL;
 
-  if (G_UNLIKELY (g_str_has_suffix (save_path, ".jpg") || g_str_has_suffix (save_path, ".jpeg")))
-    type = "jpeg";
-  else if (G_UNLIKELY (g_str_has_suffix (save_path, ".bmp")))
-    type = "bmp";
-  else if (G_UNLIKELY (g_str_has_suffix (save_path, ".webp")))
-    type = "webp";
-
-  if (G_UNLIKELY (!gdk_pixbuf_save (screenshot, save_path, type, &error, NULL)))
+  for (ImageFormat *format = screenshooter_get_image_formats (); format->type != NULL; format++)
     {
-      if (error)
+      if (!format->supported) continue;
+
+      if (screenshooter_image_format_match_extension (format, save_path))
         {
-          /* See bug #8443, looks like error is not set when path is NULL */
-          screenshooter_error ("%s", error->message);
-          g_error_free (error);
+          type = format->type;
+          option_keys = format->option_keys;
+          option_values = format->option_values;
+          break;
         }
-
-      g_free (save_path);
-
-      return NULL;
     }
-  else
+
+  /* Restrict file permission if not saved in a user-owned directory */
+  screenshooter_restrict_file_permission (save_file);
+
+  if (G_LIKELY (gdk_pixbuf_savev (screenshot, save_path, type, option_keys, option_values, &error)))
     return save_path;
+
+  if (error)
+    {
+      /* See bug #8443, looks like error is not set when path is NULL */
+      screenshooter_error ("%s", error->message);
+      g_error_free (error);
+    }
+
+  g_free (save_path);
+
+  return NULL;
 }
 
 static void
@@ -750,8 +760,7 @@ cb_dialog_response (GtkWidget *dialog, gint response, ScreenshotData *sd)
   else
     {
       gtk_widget_destroy (dialog);
-      if (!sd->plugin)
-        gtk_main_quit ();
+      sd->finalize_callback (FALSE, sd->finalize_callback_data);
     }
 }
 
@@ -1177,7 +1186,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                     G_CALLBACK (cb_save_toggled), sd);
   g_signal_connect (G_OBJECT (radio), "activate",
                     G_CALLBACK (cb_radiobutton_activate), dlg);
-  gtk_widget_set_tooltip_text (radio, _("Save the screenshot to a PNG file"));
+  gtk_widget_set_tooltip_text (radio, _("Save the screenshot to a file"));
   gtk_grid_attach (GTK_GRID (actions_grid), radio, 0, 0, 1, 1);
 
   /* Show in folder checkbox */
@@ -1191,8 +1200,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   gtk_widget_set_tooltip_text (checkbox, _("Shows the saved file in the folder"));
   gtk_grid_attach (GTK_GRID (actions_grid), checkbox, 0, 1, 1, 1);
 
-  if (sd->plugin ||
-      gdk_display_supports_clipboard_persistence (gdk_display_get_default ()))
+  if (gdk_display_supports_clipboard_persistence (gdk_display_get_default ()))
     {
       /* Copy to clipboard radio button */
       radio =
@@ -1425,12 +1433,12 @@ gchar
     gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser), filename);
 
     combobox = gtk_combo_box_text_new ();
-    gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "png", _("PNG File"));
-    gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "jpg", _("JPG File"));
-    gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "bmp", _("BMP File"));
 
-    if (screenshooter_is_format_supported ("webp"))
-      gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), "webp", _("WebP File"));
+    for (ImageFormat *format = screenshooter_get_image_formats (); format->type != NULL; format++)
+      {
+        if (!format->supported) continue;
+        gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (combobox), format->extensions[0], format->name);
+      }
 
     gtk_combo_box_set_active_id (GTK_COMBO_BOX (combobox), extension);
     g_signal_connect (combobox, "changed", G_CALLBACK (cb_combo_file_extension_changed), chooser);
